@@ -207,7 +207,16 @@ and decrypt_stream
       if is_identity no_encrypt_metadata pdf d then stream else
         let data' =
           let rec f unhashed_key key data =
-            let crypt = Pdf.ToDecrypt {Pdf.crypt_type; file_encryption_key; obj; gen; key = unhashed_key; keylength; r} in
+            let crypt =
+              Pdf.ToDecrypt
+                {Pdf.crypt_type = crypt_type;
+                 Pdf.file_encryption_key = file_encryption_key;
+                 Pdf.obj = obj;
+                 Pdf.gen = gen;
+                 Pdf.key = unhashed_key;
+                 Pdf.keylength = keylength;
+                 Pdf.r = r}
+            in
             match data with
               Pdf.Got data ->
                 (*Printf.printf "decrypt_stream: Got, encrypt = %b\n" encrypt;*)
@@ -247,7 +256,13 @@ and decrypt_stream
                   (* Otherwise, do the deferred decryption magic *)
                   let crypt =
                     Pdf.ToDecrypt
-                      {Pdf.crypt_type; file_encryption_key; obj; gen; key; keylength; r}
+                      {Pdf.crypt_type = crypt_type;
+                       Pdf.file_encryption_key = file_encryption_key;
+                       Pdf.obj = obj;
+                       Pdf.gen = gen;
+                       Pdf.key = key;
+                       Pdf.keylength = keylength;
+                       Pdf.r = r}
                   in
                     Pdf.ToGet
                       (Pdf.toget ~crypt
@@ -863,10 +878,11 @@ let encrypt_pdf_40bit_inner owner user p user_pw id pdf =
   in
     match process_cryption false true pdf (Pdfcryptprimitives.ARC4 (40, 2)) user_pw 2 user owner p id 40 None with
     | Some pdf ->
-        {pdf with
-          Pdf.trailerdict =
-            Pdf.add_dict_entry
-              pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+        let crypt_dict_num = Pdf.addobj pdf crypt_dict in
+          {pdf with
+            Pdf.trailerdict =
+              Pdf.add_dict_entry
+                pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect crypt_dict_num)}
     | None -> raise (Pdf.PDFError "Encryption 40 failed")
 
 let encrypt_pdf_40bit user_pw owner_pw banlist pdf =
@@ -890,9 +906,10 @@ let encrypt_pdf_128bit_inner owner user p user_pw id pdf =
   in
     match process_cryption false true pdf (Pdfcryptprimitives.ARC4 (128, 3)) user_pw 3 user owner p id 128 None with
     | Some pdf ->
-        {pdf with
-          Pdf.trailerdict =
-            Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+        let crypt_dict_num = Pdf.addobj pdf crypt_dict in
+          {pdf with
+            Pdf.trailerdict =
+              Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect crypt_dict_num)}
     | None -> raise (Pdf.PDFError "Encryption 128 failed")
 
 let encrypt_pdf_128bit_inner_r4 owner user p user_pw id pdf encrypt_metadata =
@@ -918,9 +935,10 @@ let encrypt_pdf_128bit_inner_r4 owner user p user_pw id pdf encrypt_metadata =
   in
     match process_cryption (not encrypt_metadata) true pdf (Pdfcryptprimitives.ARC4 (128, 4)) user_pw 4 user owner p id 128 None with
     | Some pdf ->
-        {pdf with
-          Pdf.trailerdict =
-            Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+        let crypt_dict_num = Pdf.addobj pdf crypt_dict in
+          {pdf with
+            Pdf.trailerdict =
+              Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect crypt_dict_num)}
     | None -> raise (Pdf.PDFError "Encryption 128 r4 failed")
 
 let encrypt_pdf_128bit user_pw owner_pw banlist pdf =
@@ -957,9 +975,10 @@ let encrypt_pdf_AES_inner owner user p user_pw id encrypt_metadata pdf =
         (not encrypt_metadata) true pdf Pdfcryptprimitives.AESV2 user_pw 4 user owner p id 128 None
     with
     | Some pdf ->
-        {pdf with
-          Pdf.trailerdict =
-            Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+        let crypt_dict_num = Pdf.addobj pdf crypt_dict in
+          {pdf with
+            Pdf.trailerdict =
+              Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect crypt_dict_num)}
     | None -> raise (Pdf.PDFError "Encryption AES failed")
 
 let encrypt_pdf_AES encrypt_metadata user_pw owner_pw banlist pdf =
@@ -999,9 +1018,10 @@ let encrypt_pdf_AES256_inner iso encrypt_metadata owner user p perms oe ue id ke
         (not encrypt_metadata) true pdf (Pdfcryptprimitives.AESV3 iso) "" (if iso then 6 else 5) user owner p id 256 (Some key)
     with
     | Some pdf ->
-        {pdf with
-          Pdf.trailerdict =
-            Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" crypt_dict}
+        let crypt_dict_num = Pdf.addobj pdf crypt_dict in
+          {pdf with
+            Pdf.trailerdict =
+              Pdf.add_dict_entry pdf.Pdf.trailerdict "/Encrypt" (Pdf.Indirect crypt_dict_num)}
     | None -> raise (Pdf.PDFError "256 bit Encryption AES failed")
 
 (* Algorithm 3.10 *)
