@@ -1,4 +1,5 @@
-(** Parsing and Evaluating PDF functions *)
+(** Parsing and Evaluating PDF Functions. *)
+
 type calculator =
   | If of calculator list
   | IfElse of calculator list * calculator list
@@ -58,7 +59,7 @@ and interpolation =
    n : float}
 
 and stitching =
-  {functions : pdf_fun list;
+  {functions : t list;
    bounds : float list;
    stitch_encode : float list}
 
@@ -68,26 +69,25 @@ and pdf_fun_kind =
   | Sampled of sampled
   | Calculator of calculator list
 
-and pdf_fun =
+and t =
   {func : pdf_fun_kind;
    domain : float list;
    range : float list option}
 (** The type of functions. *)
 
 (** Parse a function given a document and function object. *)
-val parse_function : Pdf.pdfdoc -> Pdf.pdfobject -> pdf_fun
-
+val parse_function : Pdf.t -> Pdf.pdfobject -> t
 
 (** Raised from [eval_function] (see below) in the case of inputs which don't
 match the evaluation *)
 exception BadFunctionEvaluation of string
 
 (** Evaluate a function given a list of inputs. *)
-val eval_function : pdf_fun -> float list -> float list
+val eval_function : t -> float list -> float list
 
 (** Flatten a function to its PDF representation *)
-val pdfobject_of_function : Pdf.pdfdoc -> pdf_fun -> Pdf.pdfobject
+val pdfobject_of_function : Pdf.t -> t -> Pdf.pdfobject
 
 (**/**)
-val print_function : pdf_fun -> unit
+val print_function : t -> unit
 
