@@ -1,6 +1,8 @@
 open Pdfutil
 open Pdfio
 
+let debug = ref false
+
 (* Graphics operators. *)
 type t =
   | Op_w of float (* Set line width *)
@@ -292,18 +294,6 @@ let lex_name i =
   nudge i;
   Obj (Pdfgenlex.LexName ("/" ^ Pdfread.getuntil_white_or_delimiter_string i))
 
-(* Lex a number *)
-let lex_number i =
-  match Pdfread.lex_number i with
-  | Pdfgenlex.LexReal r -> Obj (Pdfgenlex.LexReal r)
-  | Pdfgenlex.LexInt i -> Obj (Pdfgenlex.LexReal (float_of_int i))
-  | _ -> raise LexingEnd
-
-(* Lex and parse a dictionary to a Pdf.pdfobject. This constitutes a single
-lexeme in terms of this module. *)
-let get_dictionary i =
-  PdfObj (snd (Pdfread.parse (Pdfread.lex_dictionary i)))
-
 (* This is raised when we can't deal with some content. This should only happen
 in the case of a malformed operator stream, not on any legitimate content. *)
 exception Couldn'tHandleContent
@@ -317,6 +307,18 @@ let nocontent i =
     end;
   raise Couldn'tHandleContent
 
+(* Lex a number *)
+let lex_number i =
+  match Pdfread.lex_number i with
+  | Pdfgenlex.LexReal r -> Obj (Pdfgenlex.LexReal r)
+  | Pdfgenlex.LexInt i -> Obj (Pdfgenlex.LexReal (float_of_int i))
+  | _ -> nocontent i
+
+(* Lex and parse a dictionary to a Pdf.pdfobject. This constitutes a single
+lexeme in terms of this module. *)
+let get_dictionary i =
+  PdfObj (snd (Pdfread.parse (Pdfread.lex_dictionary i)))
+
 (* Given a colourspace and the number of bits per component, give the number of
 bytes per pixel in the stored image data. *)
 let rec components pdf resources t =
@@ -341,8 +343,9 @@ let rec components pdf resources t =
       | Some (Pdf.Integer n) -> n
       | _ -> raise (Pdf.PDFError "Bad iccstream")
       end
-  | Pdf.Array (Pdf.Name "/DeviceN"::_::alternate::_) ->
-      components pdf resources (Pdf.direct pdf alternate)
+  | Pdf.Array (Pdf.Name "/DeviceN"::Pdf.Array items::_) ->
+      (* 4th July 2017. Changed from looking at alternate to counting items. *)
+      length items
   | Pdf.Array [Pdf.Name "/Separation"; _; _; _]
   | Pdf.Array (Pdf.Name ("/Indexed" | "/I")::_::_) -> 1
   | Pdf.Array [Pdf.Name "/CalRGB"; _] -> 3
@@ -356,6 +359,7 @@ let rec components pdf resources t =
 
 (* Lex an inline image. We read the dictionary, and then the stream. *)
 let lex_inline_image pdf resources i =
+  if !debug then Printf.eprintf "lex_inline_image at %i\n" (i.pos_in ());
   try
   let dict =
     let lexemes = Pdfread.lex_dictionary i in
@@ -363,6 +367,7 @@ let lex_inline_image pdf resources i =
         (Pdfread.parse
           ([Pdfgenlex.LexLeftDict] @ lexemes @ [Pdfgenlex.LexRightDict]))
   in
+    if !debug then Printf.eprintf "dict was %s\n" (Pdfwrite.string_of_pdf dict);
     (* Read ID token *)
     Pdfread.dropwhite i;
     let c = char_of_int (i.input_byte ()) in
@@ -380,13 +385,16 @@ let lex_inline_image pdf resources i =
           not (filterspecial filters)
       in
         if toskip then ignore (i.input_byte ());
+        if !debug then Printf.eprintf "**got ID header, skipped possble byte";
         let bytes =
           let bpc =
             match
               Pdf.lookup_direct_orelse pdf "/BPC" "/BitsPerComponent" dict
             with
             | Some (Pdf.Integer bpc) -> bpc
-            | _ -> nocontent i
+            | _ ->
+                Printf.eprintf "no BPC\n";
+                nocontent i
           in
           let cspace =
             match Pdf.lookup_direct_orelse pdf "/CS" "/ColorSpace" dict with
@@ -396,13 +404,18 @@ let lex_inline_image pdf resources i =
             | Some (Pdf.Name ("/G" | "/RGB" | "/CMYK") as n) -> n
             | Some ((Pdf.Array _) as n) -> n
             | Some (Pdf.Name cspace) ->
+                if !debug then Printf.eprintf "resources is %s\n" (Pdfwrite.string_of_pdf resources);
                 begin match Pdf.lookup_direct pdf "/ColorSpace" resources with
                 | Some (Pdf.Dictionary _ as d) ->
                     begin match Pdf.lookup_direct pdf cspace d with
                     | Some c -> c
-                    | _ -> nocontent i
+                    | _ ->
+                        Printf.eprintf "no colourspace A\n";
+                        nocontent i
                     end
-                | _ -> nocontent i
+                | _ ->
+                    Printf.eprintf "no colourspace B\n";
+                    nocontent i
                 end
             | None ->
                 (* Could it be an image mask? *)
@@ -410,17 +423,25 @@ let lex_inline_image pdf resources i =
                   Pdf.lookup_direct_orelse pdf "/IM" "/ImageMask" dict
                 with
                 | Some (Pdf.Boolean true) -> Pdf.Name "/DeviceGray"
-                | _ -> nocontent i
+                | _ ->
+                    Printf.eprintf "no colourspace C\n";
+                    nocontent i
                 end
-            | _ -> nocontent i
+            | _ ->
+                Printf.eprintf "no colourspace D\n";
+                nocontent i
           in let width =
             match Pdf.lookup_direct_orelse pdf "/W" "/Width" dict with
             | Some (Pdf.Integer w) -> w
-            | _ -> nocontent i 
+            | _ ->
+                Printf.eprintf "no or malformed /W";
+                nocontent i 
           in let height =
             match Pdf.lookup_direct_orelse pdf "/H" "/Height" dict with
             | Some (Pdf.Integer h) -> h
-            | _ -> nocontent i
+            | _ ->
+                Printf.eprintf "no or malformed /H";
+                nocontent i
           in
             let bitwidth =
               components pdf resources cspace * bpc * width
@@ -444,12 +465,19 @@ let lex_inline_image pdf resources i =
                 with
                 | e -> Printf.eprintf "%s" (Printexc.to_string e); raise e
                 end
-            | Some (Pdf.Name ("/DCT" | "/DCTDecode")) ->
-                Pdfjpeg.get_jpeg_data i
+            | Some (Pdf.Name ("/DCT" | "/DCTDecode") | Pdf.Array [Pdf.Name ("/DCT" | "/DCTDecode")]) ->
+                (* FIXME. The case of DCT combined with another one is possible e.g ["/DCT"; "/A85"]. Need to re-work. But have not seen an example yet. *)
+                begin try Pdfjpeg.get_jpeg_data i with
+                  e ->
+                    Printf.eprintf "Couldn't read inline image JPEG data %s\n" (Printexc.to_string e);
+                    raise e
+                end
             | Some _ ->
                 try
                   match Pdfcodec.decode_from_input i dict with
-                  | None -> nocontent i
+                  | None ->
+                      Printf.eprintf "decode_from_input failed\n";
+                      nocontent i
                   | Some data -> data 
                 with
                   | Pdfcodec.DecodeNotSupported d ->
@@ -463,29 +491,35 @@ let lex_inline_image pdf resources i =
             Pdfread.dropwhite i;
             let c = char_of_int (i.input_byte ()) in
               let c' = char_of_int (i.input_byte ()) in
-                begin match c, c' with
-                | 'E', 'I' ->
-                    (* Remove filter, predictor, if it wasn't JPEG. *)
-                    let dict' =
-                      match
-                        Pdf.lookup_direct_orelse
-                        (Pdf.empty ()) "/F" "/Filter" dict
-                      with
-                      | Some (Pdf.Name ("/DCT" | "/DCTDecode")) -> dict
-                      | _ -> 
-                          fold_left
-                            Pdf.remove_dict_entry
-                            dict
-                            ["/Filter"; "/F"; "/DecodeParms"; "/DP"] 
-                    in
-                      dict', data
-                | x, y ->
-                   Printf.eprintf "bad end to inline image %C, %C\n" x y;
-                   nocontent i
-                end
-    | _ -> nocontent i
+                if c <> 'E' || c' <> 'I' then
+                  begin
+                    Printf.eprintf "warning: bad end to inline image %C, %C\n" c c';
+                    (* We try to find "EI" anyway, in case there is just some junk.
+                     * To do this, we drop any whitespace and any E or I character. *)
+                    Pdfread.ignoreuntil true (fun x -> Pdf.is_not_whitespace x && x <> 'E' && x <> 'I') i;
+                  end;
+                (* Remove filter, predictor, if it wasn't JPEG. *)
+                let dict' =
+                  match
+                    Pdf.lookup_direct_orelse
+                    (Pdf.empty ()) "/F" "/Filter" dict
+                  with
+                  (* FIXME as above *)
+                  | Some (Pdf.Name ("/DCT" | "/DCTDecode") | Pdf.Array [Pdf.Name ("/DCT" | "/DCTDecode")]) -> dict
+                  | _ -> 
+                      fold_left
+                        Pdf.remove_dict_entry
+                        dict
+                        ["/Filter"; "/F"; "/DecodeParms"; "/DP"] 
+                in
+                  dict', data
+    | _ ->
+        Printf.eprintf "Did not recognise beginning of inline image ID\n";
+        nocontent i
   with
-    _ -> nocontent i
+    e ->
+      Printf.eprintf "inline image reading failed: %s\n" (Printexc.to_string e);
+      nocontent i
 
 (* Lex a keyword. *)
 let lex_keyword pdf resources i =
@@ -493,28 +527,29 @@ let lex_keyword pdf resources i =
   | "true" -> Obj (Pdfgenlex.LexBool true)
   | "false" -> Obj (Pdfgenlex.LexBool false)
   | "BI" -> LexInlineImage (lex_inline_image pdf resources i)
-  | "ID" | "EI" -> raise LexingEnd (* lex_inline_image should consume these *)
-  | "" -> raise LexingEnd
+  | "ID" | "EI" -> nocontent i (* lex_inline_image should consume these *)
+  | "" -> nocontent i
   | opstring -> Op opstring
 
 (* Lex a string. *)
 let lex_string i =
   match Pdfread.lex_string i with
   | Pdfgenlex.LexString str -> Obj (Pdfgenlex.LexString str)
-  | _ -> raise LexingEnd
+  | _ -> nocontent i
 
 (* Lex a hexadecimal string. *)
 let lex_hexstring i =
   match Pdfread.lex_hexstring i with
   | Pdfgenlex.LexString str -> Obj (Pdfgenlex.LexString str)
-  | _ -> raise LexingEnd
+  | _ -> nocontent i
 
 (* Lex one token *)
 let lex_next pdf resources i =
   try
     Pdfread.dropwhite i;
     match peek_byte i with
-    | x when x = Pdfio.no_more -> raise LexingEnd
+    | x when x = Pdfio.no_more ->
+        raise LexingEnd
     | chr ->
         match char_of_int chr with
         | '/' -> lex_name i
@@ -533,7 +568,8 @@ let lex_next pdf resources i =
   with
     | Pdf.PDFError r -> 
         raise (Pdf.PDFError ("Pdfpages.lex_next => " ^ r))
-    | Failure _ (*"unopt"*) | End_of_file -> raise LexingEnd 
+    | Failure _ (*"unopt"*) | End_of_file ->
+        raise LexingEnd 
     | Couldn'tHandleContent ->
         raise (Pdf.PDFError "Malformed page content")
 
@@ -729,11 +765,13 @@ let parse_operator compatibility = function
               begin match rev r with
               | Obj Pdfgenlex.LexLeftSquare::t ->
                   let elements =
-                    map
+                    option_map
                       (function
-                       | (Obj (Pdfgenlex.LexReal i)) -> Pdf.Real i
-                       | (Obj (Pdfgenlex.LexString s)) -> Pdf.String s
-                       | _ -> raise (Pdf.PDFError "malformed TJ elt"))
+                       | (Obj (Pdfgenlex.LexReal i)) -> Some (Pdf.Real i)
+                       | (Obj (Pdfgenlex.LexString s)) -> Some (Pdf.String s)
+                       | e ->
+                           Printf.eprintf "Warning: malformed TJ element; skipping\n";
+                           None)
                       t
                   in
                     Op_TJ (Pdf.Array elements)
@@ -741,7 +779,7 @@ let parse_operator compatibility = function
               end
           | Op _::_ as l -> Op_Unknown (string_of_lexemes l)
           | l ->
-             Printf.eprintf "Empty or malformed graphics operation.";
+             Printf.eprintf "Empty or malformed graphics operation %s.\n" (string_of_lexemes l);
              Op_Unknown (string_of_lexemes l)
       in
         r, more
