@@ -4,6 +4,15 @@ top level, so their names are considered reserved words in other modules.
 
 All functions in this module are tail-recursive, unless otherwise noted. *)
 
+let rec position_gen n e = function
+  [] -> None
+| e'::t when e = e' -> Some n
+| _::t -> position_gen (n + 1) e t
+
+let position e l = position_gen 0 e l
+
+let position_1 e l = position_gen 1 e l
+
 (* Replace all instances of x with x' in s *)
 let string_replace_all x x' s =
   if x = "" then s else
@@ -245,19 +254,17 @@ let explode s =
 
 (* Make a string from a list of characters, preserving order. *)
 let implode l =
-  let s = String.create (length l) in
+  let s = Bytes.create (length l) in
     let rec list_loop x = function
        [] -> ()
-     | i::t -> String.unsafe_set s x i; list_loop (x + 1) t
+     | i::t -> Bytes.unsafe_set s x i; list_loop (x + 1) t
     in
       list_loop 0 l;
-      s
+      Bytes.to_string s
 
 (* String of character. *)
 let string_of_char c =
-  let s = String.create 1 in
-    String.unsafe_set s 0 c;
-    s
+  String.make 1 c
 
 (* Long-integer function abbreviations *)
 let i32ofi = Int32.of_int
@@ -1160,6 +1167,11 @@ let hashtable_of_dictionary pairs =
     iter (fun (k, v) -> Hashtbl.add table k v) pairs;
     table
 
+let hashset_of_list l =
+  let table = Hashtbl.create (length l * 2) in
+    iter (fun k -> Hashtbl.add table k ()) l;
+    table
+
 (* Round a number. *)
 let round x =
   let c = ceil x in let f = floor x in
